#include <cmath>

#include "glbasicshape.hpp"

using namespace std;

namespace DECS{
  namespace GL{
    void GLLine::setPoint(GLPosition beginPoint, GLPosition endPoint){
      this->beginPoint_ = beginPoint;
      this->endPoint_ = endPoint;
    }

    void GLLine::setColor(GLColor color){
      this->color_ = color;
    }

    void GLLine::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->beginPoint_.x(), this->beginPoint_.y(), 0.0);
      glVertex3d(this->endPoint_.x(), this->endPoint_.y(), 0.0);
      glEnd();
    }


    void GLTriangle::setPoint(GLPosition point1, GLPosition point2, GLPosition point3)
    {
      this->point1_ = point1;
      this->point2_ = point2;
      this->point3_ = point3;
    }

    void GLTriangle::setColor(GLColor color)
    {
      this->color_ = color;
    }

    void GLTriangle::draw()
    {
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->point1_.x(), this->point1_.y(), 0.0);
      glVertex3d(this->point2_.x(), this->point2_.y(), 0.0);
      glVertex3d(this->point3_.x(), this->point3_.y(), 0.0);
      glEnd();
    }

    void GLSquare::setPoint(GLPosition point1, GLPosition point2,
			    GLPosition point3, GLPosition point4)
    {
      this->point1_ = point1;
      this->point2_ = point2;
      this->point3_ = point3;
      this->point4_ = point4;
    }

    void GLSquare::setColor(GLColor color)
    {
      this->color_ = color;
    }

    void GLSquare::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      glVertex3d(this->point1_.x(), this->point1_.y(), 0.0);
      glVertex3d(this->point2_.x(), this->point2_.y(), 0.0);
      glVertex3d(this->point3_.x(), this->point3_.y(), 0.0);
      glVertex3d(this->point4_.x(), this->point4_.y(), 0.0);
      glEnd();
    }


    void GLCircle::setCenter(GLPosition center)
    {
      this->center_ = center;
    }

    void GLCircle::setRadius(GLdouble radius)
    {
      this->radius_ = radius;
    }

    void GLCircle::setNumberOfPoints(int nPoint)
    {
      this->nPoint_ = nPoint;
    }

    void GLCircle::setColor(GLColor color){
      this->color_ = color;
    }

    void GLCircle::draw(){
      glColor3d(this->color_.r(), this->color_.g(), this->color_.b());
      glBegin(this->mode_);
      for(int i = 0; i < this->nPoint_; i++){
	double rate = (double)i / nPoint_;
	double x = this->radius_ * 2 * cos(2.0 * M_PI * rate)
	  + this->center_.x();
	double y = this->radius_ * 2 * sin(2.0 * M_PI * rate)
	  + this->center_.y();
	glVertex3d(x, y, 0.0);
      }
      glEnd();
    }
  }
}
